@@ -2,8 +2,12 @@ import numpy as np
 import copy
 import json
 
-from spotPython.utils.convert import class_for_name
-from spotPython.utils.transform import transform_hyper_parameter_values
+from spotPython.hyperparameters.prepare import (
+    transform_hyper_parameter_values,
+    get_dict_with_levels_and_types,
+    convert_keys,
+    iterate_dict_values,
+)
 
 
 def assign_values(X: np.array, var_list: list):
@@ -32,61 +36,6 @@ def assign_values(X: np.array, var_list: list):
     return result
 
 
-def iterate_dict_values(var_dict: dict):
-    """
-    This function takes a dictionary of variables as input arguments and returns an iterator that
-    yields the values from the arrays in the dictionary.
-
-    Parameters:
-        var_dict (dict): A dictionary where keys are variable names and values are numpy arrays.
-
-    Returns:
-        iterator: An iterator that yields the values from the arrays in the dictionary.
-
-    Example:
-        >>> import numpy as np
-        >>> var_dict = {'a': np.array([1, 3, 5]), 'b': np.array([2, 4, 6])}
-        >>> for values in iterate_dict_values(var_dict):
-        ...     print(values)
-        {'a': 1, 'b': 2}
-        {'a': 3, 'b': 4}
-        {'a': 5, 'b': 6}
-    """
-    n = len(next(iter(var_dict.values())))
-    for i in range(n):
-        yield {key: value[i] for key, value in var_dict.items()}
-
-
-def convert_keys(d: dict, var_type: list):
-    """
-    Convert values in a dictionary to integers based on a list of variable types.
-
-    This function takes a dictionary `d` and a list of variable types `var_type` as arguments.
-    For each key in the dictionary,
-    if the corresponding entry in `var_type` is not equal to `"num"`,
-    the value associated with that key is converted to an integer.
-
-    Args:
-        d (dict): The input dictionary.
-        var_type (list): A list of variable types. If the entry is not `"num"` the corresponding
-        value will be converted to the type `"int"`.
-
-    Returns:
-        dict: The modified dictionary with values converted to integers based on `var_type`.
-
-    Example:
-        >>> d = {'a': '1.1', 'b': '2', 'c': '3.1'}
-        >>> var_type = ["int", "num", "int"]
-        >>> convert_keys(d, var_type)
-        {'a': 1, 'b': '2', 'c': 3}
-    """
-    keys = list(d.keys())
-    for i in range(len(keys)):
-        if var_type[i] not in ["num", "float"]:
-            d[keys[i]] = int(d[keys[i]])
-    return d
-
-
 def modify_hyper_parameter_levels(fun_control, hyperparameter, levels):
     """
 
@@ -135,105 +84,6 @@ def modify_hyper_parameter_bounds(fun_control, hyperparameter, bounds):
     return fun_control
 
 
-def get_dict_with_levels_and_types(fun_control, v):
-    """Get dictionary with levels and types.
-    The function is maps the numerical output of the hyperparameter optimization to the corresponding levels
-    of the hyperparameter needed by the core model, i.e., the tuned algorithm.
-    The function takes the dictionaries d and v and returns a new dictionary with the same keys as v
-    but with the values of the levels of the keys from d.
-    If the key value in the dictionary is 0, it takes the first value from the list,
-    if it is 1, it takes the second and so on.
-    If a key is not in d, it takes the key from v.
-    If the core_model_parameter_type value is instance, it returns the class of the value from the module
-    via getattr("class", value).
-    For example,
-    if d = {"HoeffdingTreeRegressor":{
-        "leaf_prediction": {
-            "levels": ["mean", "model", "adaptive"],
-            "type": "factor",
-            "default": "mean",
-            "core_model_parameter_type": "str"},
-        "leaf_model": {
-            "levels": ["linear_model.LinearRegression", "linear_model.PARegressor", "linear_model.Perceptron"],
-            "type": "factor",
-            "default": "LinearRegression",
-            "core_model_parameter_type": "instance"},
-            "splitter": {"levels": ["EBSTSplitter", "TEBSTSplitter", "QOSplitter"],
-            "type": "factor",
-            "default": "EBSTSplitter", "core_model_parameter_type": "instance()"},
-        "binary_split": {
-            "levels": [0, 1],
-            "type": "factor",
-            "default": 0,
-            "core_model_parameter_type": "bool"},
-        "stop_mem_management": {
-            "levels": [0, 1],
-            "type": "factor",
-            "default": 0,
-            "core_model_parameter_type": "bool"}}}
-        and
-        v = {'grace_period': 200,
-            'max_depth': 10,
-            'delta': 1e-07,
-            'tau': 0.05,
-            'leaf_prediction': 0,
-            'leaf_model': 0,
-            'model_selector_decay': 0.95,
-            'splitter': 1,
-            'min_samples_split': 9,
-            'binary_split': 0,
-            'max_size': 500.0}
-        then the function returns
-        {'grace_period': 200,
-        'max_depth': 10,
-        'delta': 1e-07,
-        'tau': 0.05,
-        'leaf_prediction': 'mean',
-        'leaf_model': linear_model.LinearRegression,
-        'model_selector_decay': 0.95,
-        'splitter': 'TEBSTSplitter',
-        'min_samples_split': 9,
-        'binary_split': 0,
-        'max_size': 500.0}.
-
-    Args:
-        fun_control (dict): dictionary with levels and types
-        v (dict): dictionary with values
-
-    Returns:
-        new_dict (dict): dictionary with levels and types
-
-    Example:
-        >>> d = {"HoeffdingTreeRegressor":{
-                "leaf_prediction": {"levels": ["mean", "model", "adaptive"],
-                                    "type": "factor",
-                                    "default": "mean",
-                                    "core_model_parameter_type": "str"}}}
-            v = {"leaf_prediction": 0}
-            get_dict_with_levels_and_types(d, v)
-            {"leaf_prediction": "mean"}
-    """
-    d = fun_control["core_model_hyper_dict"]
-    new_dict = {}
-    for key, value in v.items():
-        if key in d and d[key]["type"] == "factor":
-            if d[key]["core_model_parameter_type"] == "instance":
-                if "class_name" in d[key]:
-                    mdl = d[key]["class_name"]
-                c = d[key]["levels"][value]
-                new_dict[key] = class_for_name(mdl, c)
-            elif d[key]["core_model_parameter_type"] == "instance()":
-                mdl = d[key]["class_name"]
-                c = d[key]["levels"][value]
-                k = class_for_name(mdl, c)
-                new_dict[key] = k()
-            else:
-                new_dict[key] = d[key]["levels"][value]
-        else:
-            new_dict[key] = v[key]
-    return new_dict
-
-
 def get_default_values(fun_control):
     """Get the values from the "default" keys from the dictionary fun_control as a list.
     If the key of the value has as "type" the value "int" or "float", convert the value to the corresponding type.
@@ -461,7 +311,7 @@ def replace_levels_with_positions(hyper_dict, hyper_dict_values):
 
 
 def get_values_from_dict(dictionary) -> np.array:
-    """Get the values from a dictionary as a list.
+    """Get the values from a dictionary as an array.
     Generate an np.array that contains the values of the keys of a dictionary
     in the same order as the keys of the dictionary.
     Args:
@@ -476,25 +326,32 @@ def get_values_from_dict(dictionary) -> np.array:
     return np.array(list(dictionary.values()))
 
 
-def get_values_from_var_dict(var_dict: dict, fun_control: dict):
+def return_conf_list_from_var_dict(var_dict: dict, fun_control: dict) -> list:
     """This function takes a dictionary of variables and a dictionary of function control.
     Args:
         var_dict (dict): A dictionary of variables.
         fun_control (dict): A dictionary of function control.
     Returns:
-        dict: A dictionary of hyper parameter values. Transformations are applied to the values.
-    Example:
+        list A list of dictionaries of hyper parameter values. Transformations are applied to the values.
+    Examples:
         >>> import numpy as np
-        >>> var_dict = {'a': np.array([1, 3, 5]), 'b': np.array([2, 4, 6])}
-        >>> fun_control = {'var_type': ['int', 'int']}
-        >>> get_values_from_var_dict(var_dict, fun_control)
-        {'a': [1, 3, 5], 'b': [2, 4, 6]}
+            var_dict = {'a': np.array([1]),
+                        'b': np.array([2])}
+            fun_control = {'var_type': ['int', 'int']}
+            return_conf_list_from_var_dict(var_dict, fun_control)
+            var_dict = {'a': np.array([1, 3, 5]), 'b': np.array([2, 4, 6])}
+            fun_control = {'var_type': ['int', 'int']}
+            return_conf_list_from_var_dict(var_dict, fun_control)
+            {'a': [1, 3, 5], 'b': [2, 4, 6]}
+
     """
+    conf_list = []
     for values in iterate_dict_values(var_dict):
         values = convert_keys(values, fun_control["var_type"])
         values = get_dict_with_levels_and_types(fun_control=fun_control, v=values)
         values = transform_hyper_parameter_values(fun_control=fun_control, hyper_parameter_values=values)
-    return values
+        conf_list.append(values)
+    return conf_list
 
 
 def add_core_model_to_fun_control(core_model, fun_control, hyper_dict, filename) -> dict:
@@ -503,7 +360,8 @@ def add_core_model_to_fun_control(core_model, fun_control, hyper_dict, filename)
         core_model (class): The core model.
         fun_control (dict): The function control dictionary.
         hyper_dict (dict): The hyper parameter dictionary.
-        filename (str): The name of the json file that contains the hyper parameter dictionary. Optional. Default is None.
+        filename (str): The name of the json file that contains the hyper parameter dictionary.
+        Optional. Default is None.
     Returns:
         (dict): The function control dictionary.
     Example:
@@ -529,7 +387,7 @@ def add_core_model_to_fun_control(core_model, fun_control, hyper_dict, filename)
 def get_default_hyperparameters_for_core_model(fun_control, hyper_dict) -> dict:
     X0 = get_default_hyperparameters_for_fun(fun_control, hyper_dict)
     var_dict = assign_values(X0, fun_control["var_name"])
-    values = get_values_from_var_dict(var_dict, fun_control)
+    values = return_conf_list_from_var_dict(var_dict, fun_control)[0]
     return values
 
 
